<!DOCTYPE html>
<html xmlns:t="http://twistedmatrix.com/ns/twisted.web.template/0.1">
  <head>
    <title>Docstrings with Formatting Problems.</title>

    <meta http-equiv="Content-Type" content="text/html;charset=utf-8" />
    <link rel="stylesheet" type="text/css" href="bootstrap.min.css" />
    <link rel="stylesheet" type="text/css" href="apidocs.css"/>
  </head>
  <body>
    <h1>Docstrings with Formatting Problems.</h1>
    <ul>
      <li t:render="problemObjects">
        <t:slot name="link" />
      </li>
    </ul>
  </body>
</html>
