<!DOCTYPE html>
<html xmlns:t="http://twistedmatrix.com/ns/twisted.web.template/0.1" t:render="all">
  <head>
    <title><t:slot name="title">Something</t:slot> : API documentation</title>

    <meta http-equiv="Content-Type" content="text/html;charset=utf-8" />
    <link rel="stylesheet" type="text/css" href="bootstrap.min.css" />
    <link rel="stylesheet" type="text/css" href="apidocs.css"/>
  </head>
  <body>

    <nav class="navbar navbar-default">
      <div class="container">
        <div class="navbar-header">
          <a class="navbar-brand" href="index.html">
            <t:slot name="project">Some Project</t:slot> API Documentation
          </a>
        </div>
      </div>
    </nav>

    <div class="container">

      <div class="page-header">
        <t:slot name="heading"><h1>Heading</h1></t:slot>

        <span id="partOf">
          <t:slot name="part">Part of something</t:slot>
          <a t:render="source">View Source</a>
          <a t:render="inhierarchy">(View In Hierarchy)</a>
        </span>
      </div>

      <div class="extrasDocstring">
        <t:slot name="extras">
          A docstring.
        </t:slot>
      </div>

      <div class="moduleDocstring">
        <t:slot name="docstring">
          A docstring.
        </t:slot>
      </div>

      <div id="splitTables">
        <t:slot name="mainTable" />
        <t:transparent t:render="baseTables">
          <p class="inheritedFrom">
            Inherited from <t:slot name="baseName" />:
          </p>
          <t:slot name="baseTable" />
          </t:transparent>

          <t:slot name="packageInitTable"> </t:slot>
      </div>

      <div id="childList">

        <t:slot name="childlist"> </t:slot>

      </div>
      <address>
        <a href="index.html">API Documentation</a> for <t:slot name="project">Some
          Project</t:slot>, generated by <a href="https://github.com/twisted/pydoctor/">pydoctor</a> at <t:slot name="buildtime">some time</t:slot>.
      </address>

    </div>
  </body>
</html>
