<!DOCTYPE html>
<html xmlns:t="http://twistedmatrix.com/ns/twisted.web.template/0.1">
  <head>
    <title t:render="title">Something</title>
    <meta http-equiv="Content-Type" content="text/html;charset=utf-8" />
    <link rel="stylesheet" type="text/css" href="bootstrap.min.css" />
    <link rel="stylesheet" type="text/css" href="apidocs.css"/>
  </head>
  <body>
    <nav class="navbar navbar-default">
      <div class="container">

        <div class="navbar-header">
          <a class="navbar-brand" href="index.html">
            <t:transparent t:render="project">Some Project</t:transparent> API Documentation
          </a>
        </div>
      </div>
    </nav>

    <div class="container">

      <div class="page-header">
        <h1 t:render="heading">Heading</h1>
      </div>

      <ul t:render="stuff">
      </ul>

    </div>
  </body>
</html>
