<div xmlns:t="http://twistedmatrix.com/ns/twisted.web.template/0.1" class="function">
  <a>
    <t:attr name="name" t:render="functionAnchor" />
  </a>
  <a>
    <t:attr name="name" t:render="shortFunctionAnchor" />
  </a>
  <div class="functionHeader">
    <t:transparent t:render="decorator" />
    def
    <t:transparent t:render="functionName">
      functionName():
    </t:transparent>
    <a class="functionSourceLink" t:render="sourceLink">
      <t:attr name="href"><t:slot name="sourceHref" /></t:attr>
      (source)
    </a>
  </div>
  <div class="docstring functionBody">
    <t:transparent t:render="functionExtras" />
    <t:transparent t:render="functionBody">
      Docstring.
    </t:transparent>
  </div>
</div>
