<table xmlns:t="http://twistedmatrix.com/ns/twisted.web.template/0.1" class="children sortable">
  <t:attr name="id" t:render="id" />
  <tr t:render="rows">
    <t:attr name="class" t:render="class_" />
    <td t:render="kind">
      Kind
    </td>
    <td t:render="name">
      Name
    </td>
    <td t:render="summaryDoc">
      Summary
    </td>
  </tr>
</table>
